Attribute VB_Name = "ModFormats"
' ModFormats
'
' Latest Update: 23 november 2011
'  - All formats now have the specified number of leading zeros
'
' Public Function TechFormat(Entry As Double, Significants As Integer) As String 'tested dec 2006
' Returns a string representing Entry in Technical Notation, that is
' like scientific notation, but with the exponent only in multiples of 3.
' Where applicable the exponent is replaced by suffixes like m for milli, M for mega etc.
' When postfixes exhaust the E-notation is used, but with E in multiples of 3.
' The number of significant digits (that's different from the number of decimal places!) can
' be selected, but less than 3 is ignored
'
' Public Sub TechFormat1(ByVal Entry As Double, Mantisse As Double, Prefix As String)
' returns
' A number of formatting functions.
' example: Fmt12 has one digit before the decimal dot and 2 decimal places.
'Public Function Fmt10(Value) As String
' etc.....

Private Const SpacesString = "                                            "

Option Explicit

Public Sub TechFormat1(ByVal Entry As Double, Mantisse As Double, Prefix As String)
' Returns separate Mantisse and Prefix.
Dim Exponent As Double
Dim IntExp As Integer
Dim NewIntExp As Integer
Dim Sign As Integer
Const Prefixes As String = "afpnum kMGTPE"
Sign = 1
If Entry < 0 Then
    Sign = -1
    Entry = -Entry
End If
If Entry > 0 Then
    Exponent = Log(Entry) / Log(10#)
    IntExp = Int(Exponent)
    If IntExp < -18 Then IntExp = -18
    If IntExp > 18 Then IntExp = 18
    NewIntExp = (IntExp - 1) / 3
    Mantisse = Entry * (10 ^ (((IntExp + 27) Mod 3) - IntExp)) * Sign
Prefix = Mid(Prefixes, NewIntExp + 7, 1)
Else
    Mantisse = 0
    Prefix = " "
End If
End Sub

Public Function TechFormat(Entry As Double, Significants As Integer) As String 'tested dec 2006
' Returns a string
Dim Exponent As Double
Dim IntExp As Integer
Dim NewIntExp As Integer
Dim Mantisse As Double
Dim Sign As Integer
Dim TempStr As String
Const Suffixes As String = "afpnum kMGTPE"
Const FmtStr = "00000000000000000000"
Sign = 1
If Entry < 0 Then 'catch negatives
    Sign = -1
    Entry = -Entry
End If
If Entry > 0 Then
    Exponent = Log(Entry) / Log(10#)
    IntExp = Int(Exponent)
    If IntExp > 0 Then
        NewIntExp = ((IntExp) \ 3) * 3  'in multiples of 3
    Else
        NewIntExp = ((IntExp - 2) \ 3) * 3
    End If
    'Multiply mantissa such that exponent is always a multiple of 3
    '333 is more than log10(Double range), so it will never be exceeded
    Mantisse = Entry * (10 ^ (((IntExp + 333) Mod 3) - IntExp)) * Sign
End If
If Significants < 3 Then Significants = 3
TempStr = Format(Mantisse, "##0." & Left(FmtStr, (NewIntExp) - (IntExp) + Significants - 1))
If (IntExp < -18) Or (IntExp > 18) Then ' we don't have literals for this
    TechFormat = TempStr & "E" & Format(NewIntExp, "##0")
Else
    TechFormat = TempStr & Mid(Suffixes, NewIntExp \ 3 + 7, 1)
End If
End Function

' TO BE TESTED
Public Function LSpaces(Source As String, NSpaces As Byte) As String
' From left: Replace a "0" by a " ", but not if it is the last char, or
' if it is followed by a decimal point
Dim N As Integer
Dim TempStr As String
Dim TempChar As String
Dim FoundPoint As Boolean
Dim DigitsBefore As Byte
TempStr = ""
FoundPoint = False
DigitsBefore = 0
For N = 1 To Len(Source)
    TempChar = Mid(Source, N, 1)
    If N < Len(Source) Then   ' never change the last char
        If Mid(Source, N + 1, 1) = "." Then
            DigitsBefore = N + 1
            FoundPoint = True
        End If
        If (TempChar = "0") And (Not FoundPoint) Then TempChar = " "
    End If
    TempStr = TempStr & TempChar
Next N
If NSpaces > N Then
    LSpaces = Mid(SpacesString, 1, NSpaces - N) & TempStr
Else
    LSpaces = TempStr
End If
End Function


'****************************************************************************
' Formats with leading Zeros
Public Function Fmt010(Value) As String
Fmt010 = Format(Value, "0")
End Function

Public Function Fmt011(Value) As String
Fmt011 = Format(Value, "0.0")
End Function

Public Function Fmt012(Value) As String
Fmt012 = Format(Value, "0.00")
End Function

Public Function Fmt013(Value) As String
Fmt013 = Format(Value, "0.000")
End Function

Public Function Fmt014(Value) As String
Fmt014 = Format(Value, "0.0000")
End Function

Public Function Fmt015(Value) As String
Fmt015 = Format(Value, "0.00000")
End Function

Public Function Fmt016(Value) As String
Fmt016 = Format(Value, "0.000000")
End Function

Public Function Fmt020(Value) As String
Fmt020 = Format(Value, "00")
End Function

Public Function Fmt021(InVal As Double) As String
Fmt021 = Format(InVal, "000.0")
End Function

Public Function Fmt022(Value) As String
Fmt022 = Format(Value, "00.00")
End Function

Public Function Fmt023(Value) As String
Fmt023 = Format(Value, "00.000")
End Function

Public Function Fmt030(Value) As String
Fmt030 = Format(Value, "000")
End Function

Public Function Fmt032(Value) As String
Fmt032 = Format(Value, "000.00")
End Function

Public Function Fmt033(Value) As String
Fmt033 = Format(Value, "000.000")
End Function

Public Function Fmt040(Value) As String
Fmt040 = Format(Value, "0000")
End Function

Public Function Fmt041(Value) As String
Fmt041 = Format(Value, "0000.0")
End Function

Public Function Fmt042(Value) As String
Fmt042 = Format(Value, "0000.00")
End Function

Public Function Fmt043(Value) As String
Fmt043 = Format(Value, "0000.000")
End Function

Public Function Fmt050(Value) As String
Fmt050 = Format(Value, "00000")
End Function

Public Function Fmt052(Value) As String
Fmt052 = Format(Value, "00000.00")
End Function

Public Function Fmt053(Value) As String
Fmt053 = Format(Value, "00000.000")
End Function

Public Function Fmt060(Value) As String
Fmt060 = Format(Value, "000000")
End Function

'****************************************************************
' Formats without leading zeros

Public Function Fmt_10(Value) As String
Fmt_10 = Format(Value, "0")
End Function

Public Function Fmt_11(Value) As String
Fmt_11 = Format(Value, "0.0")
End Function

Public Function Fmt_12(Value) As String
Fmt_12 = Format(Value, "0.00")
End Function

Public Function Fmt_13(Value) As String
Fmt_13 = Format(Value, "0.000")
End Function

Public Function Fmt_20(Value) As String
Fmt_20 = Format(Value, "#0")
End Function

Public Function Fmt_21(InVal As Double) As String
Fmt_21 = Format(InVal, "#0.0")
End Function

Public Function Fmt_22(Value) As String
Fmt_22 = Format(Value, "#0.00")
End Function

Public Function Fmt_23(Value) As String
Fmt_23 = Format(Value, "#0.000")
End Function

Public Function Fmt_30(Value) As String
Fmt_30 = Format(Value, "##0")
End Function

Public Function Fmt_32(Value) As String
Fmt_32 = Format(Value, "##0.00")
End Function

Public Function Fmt_33(Value) As String
Fmt_33 = Format(Value, "##0.000")
End Function

Public Function Fmt_40(Value) As String
Fmt_40 = Format(Value, "###0")
End Function

Public Function Fmt_41(Value) As String
Fmt_41 = Format(Value, "###0.0")
End Function

Public Function Fmt_42(Value) As String
Fmt_42 = Format(Value, "###0.00")
End Function

Public Function Fmt_43(Value) As String
Fmt_43 = Format(Value, "###0.000")
End Function

Public Function Fmt_50(Value) As String
Fmt_50 = Format(Value, "####0")
End Function

Public Function Fmt_52(Value) As String
Fmt_52 = Format(Value, "####0.00")
End Function

Public Function Fmt_53(Value) As String
Fmt_53 = Format(Value, "####0.000")
End Function

Public Function Fmt_60(Value) As String
Fmt_60 = Format(Value, "#####0")
End Function

'*************************************************************************************************
' Time format for playing audio

Function FmtM2S3(TimeInTenthSeconds As Long) As String
' 2 digit Minutes,:, 3 digit 10th Seconds all with leading zeroes
FmtM2S3 = Fmt020(Int(TimeInTenthSeconds / 600)) & ":" & Fmt030(TimeInTenthSeconds Mod 600)
End Function

' Time format for time 0f day
Function FmtH2M2S2(Tijd) As String
' 2 digit Hours, 2 digit Minutes, 2 digit Seconds, all with leading zeroes
FmtH2M2S2 = Fmt020(Hour(Tijd)) & ":" & Fmt020(Minute(Tijd)) & ":" & Fmt020(Second(Tijd))
End Function


